import js from '@eslint/js';

export default [
    js.configs.recommended,
    {
        languageOptions: {sourceType: 'script'},
        linterOptions: {reportUnusedDisableDirectives: true},

        rules: {
            // Possible problems
            'no-await-in-loop': 'error',
            'no-constant-binary-expression': 'error',
            'no-duplicate-imports': ['error', {includeExports: true}],
            'no-self-compare': 'error',
            'no-template-curly-in-string': 'error',
            'no-unreachable-loop': 'error',
            'no-unused-private-class-members': 'error',
            'no-use-before-define': 'error',

            // Suggestions
            'accessor-pairs': 'error',
            'array-callback-return': ['error', {allowImplicit: true}],
            'arrow-body-style': ['error', 'as-needed'],
            'camelcase': 'error',
            'capitalized-comments': [
                'error', 'always',
                {'ignoreInlineComments': true, 'ignoreConsecutiveComments': true},
            ],
            'consistent-this': ['error', 'self'],
            'curly': ['error', 'multi-line', 'consistent'],
            'default-case-last': 'error',
            'default-param-last': 'error',
            'dot-notation': 'error',
            'eqeqeq': 'error',
            'func-names': ['error', 'as-needed'],
            'func-style': ['error', 'declaration'],
            'grouped-accessor-pairs': ['error', 'getBeforeSet'],
            'guard-for-in': 'error',
            'id-length': 'error',
            'multiline-comment-style': ['error', 'separate-lines', {checkJSDoc: true}],
            'new-cap': 'error',
            'no-alert': 'error',
            'no-array-constructor': 'error',
            'no-caller': 'error',
            'no-confusing-arrow': 'error',
            'no-console': 'error',
            'no-constructor-return': 'error',
            'no-else-return': ['error', {allowElseIf: false}],
            'no-empty-static-block': 'error',
            'no-eq-null': 'error',
            'no-eval': 'error',
            'no-extend-native': 'error',
            'no-extra-bind': 'error',
            'no-extra-label': 'error',
            'no-floating-decimal': 'error',
            'no-implicit-coercion': ['error', {disallowTemplateShorthand: true}],
            'no-implied-eval': 'error',
            'no-invalid-this': 'error',
            'no-iterator': 'error',
            'no-label-var': 'error',
            'no-lone-blocks': 'error',
            'no-lonely-if': 'error',
            'no-loop-func': 'error',
            'no-mixed-operators': 'error',
            'no-multi-assign': 'error',
            'no-new': 'error',
            'no-new-func': 'error',
            'no-new-native-nonconstructor': 'error',
            'no-new-object': 'error',
            'no-new-wrappers': 'error',
            'no-octal-escape': 'error',
            'no-param-reassign': ['error', {props: true}],
            'no-plusplus': 'error',
            'no-promise-executor-return': 'error',
            'no-proto': 'error',
            'no-return-assign': 'error',
            'no-return-await': 'error',
            'no-script-url': 'error',
            'no-sequences': 'error',
            'no-shadow': ['error', {builtinGlobals: true}],
            'no-throw-literal': 'error',
            'no-undef-init': 'error',
            'no-undefined': 'error',
            'no-underscore-dangle': 'error',
            'no-unneeded-ternary': ['error', {defaultAssignment: false}],
            'no-unused-expressions': [
                'error',
                {
                    allowShortCircuit: false,
                    allowTernary: false,
                    allowTaggedTemplates: false,
                },
            ],
            'no-unused-vars': ['error', {vars: 'all', args: 'all'}],
            'no-useless-call': 'error',
            'no-useless-computed-key': 'error',
            'no-useless-concat': 'error',
            'no-useless-constructor': 'error',
            'no-useless-rename': 'error',
            'no-useless-return': 'error',
            'no-var': 'error',
            'no-void': 'error',
            'no-warning-comments': 'error',
            'object-shorthand': [
                'error', 'always',
                {avoidQuotes: true, ignoreConstructors: false},
            ],
            'one-var': ['error', 'never'],
            'operator-assignment': 'error',
            'prefer-arrow-callback': 'error',
            'prefer-const': ['error', {destructuring: 'any', ignoreReadBeforeAssign: true}],
            'prefer-exponentiation-operator': 'error',
            'prefer-named-capture-group': 'error',
            'prefer-numeric-literals': 'error',
            'prefer-object-has-own': 'error',
            'prefer-object-spread': 'error',
            'prefer-promise-reject-errors': ['error', {allowEmptyReject: true}],
            'prefer-regex-literals': ['error', {disallowRedundantWrapping: true}],
            'prefer-rest-params': 'error',
            'prefer-spread': 'error',
            'prefer-template': 'error',
            'quote-props': ['error', 'as-needed', {keywords: true, numbers: true}],
            'require-await': 'error',
            'require-yield': 'error',
            'sort-imports': 'error',
            'spaced-comment': 'error',
            'strict': ['error', 'global'],
            'symbol-description': 'error',
            'vars-on-top': 'error',
            'yoda': 'error',

            // Layout & formatting
            'array-bracket-newline': ['error', 'consistent'],
            'array-bracket-spacing': 'error',
            'array-element-newline': ['error', 'consistent'],
            'arrow-parens': ['error', 'always'],
            'arrow-spacing': 'error',
            'block-spacing': 'error',
            'brace-style': ['error', '1tbs', {allowSingleLine: true}],
            'comma-dangle': ['error', 'always-multiline'],
            'comma-spacing': 'error',
            'comma-style': 'error',
            'computed-property-spacing': 'error',
            'dot-location': ['error', 'property'],
            'eol-last': ['error', 'never'],
            'func-call-spacing': 'error',
            'function-call-argument-newline': ['error', 'consistent'],
            'function-paren-newline': ['error', 'consistent'],
            'generator-star-spacing': 'error',
            'implicit-arrow-linebreak': 'error',
            'indent': ['error', 4],
            'key-spacing': 'error',
            'keyword-spacing': 'error',
            'linebreak-style': ['error', 'unix'],
            'new-parens': 'error',
            'no-extra-parens': [
                'error', 'all',
                {
                    nestedBinaryExpressions: false,
                    returnAssign: false,
                    enforceForArrowConditionals: false,
                },
            ],
            'no-multi-spaces': 'error',
            'no-multiple-empty-lines': ['error', {max: 1, maxEOF: 0, maxBOF: 0}],
            'no-tabs': 'error',
            'no-trailing-spaces': 'error',
            'no-whitespace-before-property': 'error',
            'nonblock-statement-body-position': 'error',
            'object-curly-newline': 'error',
            'object-curly-spacing': 'error',
            'operator-linebreak': ['error', 'before'],
            'padded-blocks': ['error', 'never'],
            'quotes': ['error', 'single', {avoidEscape: true}],
            'rest-spread-spacing': 'error',
            'semi': ['error', 'always'],
            'semi-spacing': 'error',
            'semi-style': 'error',
            'space-before-blocks': 'error',
            'space-before-function-paren': [
                'error', {anonymous: 'always', named: 'never', asyncArrow: 'always'},
            ],
            'space-in-parens': 'error',
            'space-infix-ops': 'error',
            'space-unary-ops': 'error',
            'switch-colon-spacing': 'error',
            'template-curly-spacing': 'error',
            'template-tag-spacing': 'error',
            'unicode-bom': 'error',
            'wrap-iife': 'error',
            'wrap-regex': 'error',
            'yield-star-spacing': 'error',
        },
    },
]
